/*
 * Simple but hopefully safer RPC service replacement
 *
 * Copright (C) 2000 <okir@caldera.de>
 */

#ifndef _RPC_SVC_H
#define _RPC_SVC_H

#include <smallrpc/types.h>
#include <smallrpc/auth.h>
#include <smallrpc/xdr.h>
#include <smallrpc/rpc_msg.h>

#ifdef cplusplus
extern "C" {
#endif

/*
 * We define SVCXPRT opaquely so nobody gets tempted to
 * muck with internal data.
 */
typedef struct SVCXPRT		SVCXPRT;

/*
 * This struct holds information on the request we're just
 * processing.
 */
struct svc_req {
	u_int32_t		rq_prog;
	u_int32_t		rq_vers;
	u_int32_t		rq_proc;
	struct opaque_auth	rq_cred;
	struct opaque_auth	rq_verf;	/* non-standard addition */
	void *			rq_clntcred;
	SVCXPRT *		rq_xprt;
};

/* Get the caller's and callee's socket address */
struct sockaddr *	svc_getcaller(SVCXPRT *);
struct sockaddr *	svc_getsvcaddr(SVCXPRT *);

#ifndef __DISPATCH_FN_T
#define __DISPATCH_FN_T
typedef void		__dispatch_fn_t(struct svc_req *, SVCXPRT *);
#endif /* __DISPATCH_FN_T */

/* Some standard RPC functions we implement */
extern bool_t		svc_register(SVCXPRT *, u_int32_t, u_int32_t,
					__dispatch_fn_t, int);
extern void		svc_run();
extern void		svc_exit();
extern bool_t		svc_getargs(SVCXPRT *, xdrproc_t, void *);
extern bool_t		svc_sendreply(SVCXPRT *, xdrproc_t, void *);
extern void		svcerr_decode(SVCXPRT *);
extern void		svcerr_weakauth(SVCXPRT *);
extern void		svcerr_noprog(SVCXPRT *);
extern void		svcerr_progvers(SVCXPRT *, u_int32_t, u_int32_t);
extern void		svcerr_noproc(SVCXPRT *);
extern void		svcerr_auth(SVCXPRT *, enum auth_stat);
extern void		svcerr_systemerr(SVCXPRT *);
extern bool_t		svc_reply(SVCXPRT *, xdrproc_t, void *);
extern bool_t		svc_freeargs(SVCXPRT *, xdrproc_t, void *);
extern void		svc_destroy(SVCXPRT *);


#define RPC_ANYSOCK	-1

/*
 * Create RPC service transports.
 */
#define RPCSMALLMSGSIZE	(8192)
extern SVCXPRT *	svcudp_create(int sock);
extern SVCXPRT *	svcudp_bufcreate(int sock, unsigned int, unsigned int);
extern SVCXPRT *	svctcp_create(int sock, unsigned int, unsigned int);
extern SVCXPRT *	svcunix_create(int sock, unsigned int, unsigned int, const char *);

#ifdef cplusplus
}
#endif

#endif /* _RPC_SVC_H */
