/*
 * Simple but hopefully safer RPC service replacement
 *
 * Copright (C) 2000 <okir@caldera.de>
 */

#ifndef _RPC_XDR_H
#define _RPC_XDR_H

#include <smallrpc/types.h>

#ifdef cplusplus
extern "C" {
#endif

/*
 * This struct hides all the encoding/decoding machinery from view
 */
typedef struct XDR	XDR;

/*
 * The following XDR operations are supported
 */
enum xdr_op {
	XDR_ENCODE	= 0,
	XDR_DECODE	= 1,
	XDR_FREE	= 2
};

/*
 * The XDR struct itself
 */
struct XDR {
	enum xdr_op	x_op;
	void *		x_base;
	unsigned int	x_size;
	unsigned int	x_pos;

	void *		(*getmem)(XDR *, unsigned int);
};

/*
 * XDR encode/decode function.
 */
typedef bool_t		(*xdrproc_t)(XDR *, void *);

/*
 * List of choices for a discriminated union
 */
struct xdr_discrim {
	int		value;
	xdrproc_t *	proc;
};

extern void		xdrmem_create(XDR *, void *, unsigned int , int );
extern void		xdr_destroy(XDR *);
extern unsigned int	xdr_getpos(XDR *);
extern unsigned int	xdr_getavail(XDR *);
extern void		xdr_free(xdrproc_t , void *);
extern bool_t		xdr_void();
extern void *		xdr_inline(XDR *, unsigned int );
extern bool_t		xdr_bool(XDR *, bool_t *);
extern bool_t		xdr_enum(XDR *, int *);
extern bool_t		xdr_char(XDR *, char *);
extern bool_t		xdr_short(XDR *, short *);
extern bool_t		xdr_int(XDR *, int *);
extern bool_t		xdr_long(XDR *, long *);
extern bool_t		xdr_u_char(XDR *, unsigned char *);
extern bool_t		xdr_u_short(XDR *, unsigned short *);
extern bool_t		xdr_u_int(XDR *, unsigned int *);
extern bool_t		xdr_u_long(XDR *, unsigned long *);
extern bool_t		xdr_float(XDR *, float *);
extern bool_t		xdr_double(XDR *, double *);
extern bool_t		xdr_array(XDR *, void **, unsigned int *, unsigned int,
					unsigned int, xdrproc_t);
extern bool_t		xdr_vector(XDR *, void *, unsigned int , unsigned int , xdrproc_t );
extern bool_t		xdr_bytes(XDR *, void **, unsigned int *, unsigned int );
extern bool_t		xdr_string(XDR *, char **, unsigned int );
extern bool_t		xdr_wrapstring(XDR *, char **);
extern bool_t		xdr_opaque(XDR *, void *, unsigned int );
extern bool_t		xdr_pointer(XDR *, void **, unsigned int , xdrproc_t );
extern bool_t		xdr_reference(XDR *, void **, unsigned int , xdrproc_t );
extern bool_t		xdr_union(XDR *, int *, void *, struct xdr_discrim *, xdrproc_t);

#ifdef cplusplus
}
#endif

#endif /* _RPC_XDR_H */
